/* -------------------------------------------------------
                         tftdisplay.h

     Bibliothek fuer TFT 1.44" 128 x 128 Pixel / 16 Bit
     Color mit ILI9163 Chip (China-Display)

     MCU   :  STM8S103F3
     Takt  :  interner Takt 16 MHz

     04.02.2020  R. Seelig
   ------------------------------------------------------ */

#ifndef in_tftdisplay
  #define in_tftdisplay

  #include "stm8s.h"
  #include "stm8_init.h"
  #include "stm8_gpio.h"
  #include "spi.h"
  #include "my_printf.h"


  /* ------------------------------------------------------------------
                            Displayauswahl

       es kann (und muss) nur ein einziges Display ausgewaehlt sein.
     ------------------------------------------------------------------ */

  // ----------------------- SPI- Displays ----------------------------

  #define  ili9163                  0
  #define  ili9340                  0
  #define  st7735r                  1
  #define  s6d02a1                  0
  #define  ili9225                  0
  #define  st7789                   0

  /* ------------------------------------------------------------------
        verfuegbare Textfonts auswaehlen (auch Kombinationen erlaubt)
     ------------------------------------------------------------------ */

  #define fnt5x7_enable             0                // 1 : Font verfuegbar
                                                     // 0 : nicht verfuegbar
  #define fnt8x8_enable             1                // 1 : Font verfuegbar
                                                     // 0 : nicht verfuegbar
  #define fnt12x16_enable           0                // 1 : Font verfuegbar
                                                     // 0 : nicht verfuegbar

  #define lastascii 126                              // letztes verfuegbares Asciizeichen


/* -------------------------------------------------------------
                 Hardwareanbindung LCD Pins
   ------------------------------------------------------------- */

  #define LCD_PORT                PC_ODR
  #define LCD_CR1                 PC_CR1
  #define LCD_DDR                 PC_DDR
  #define LCD_RST_PIN             PC3
  #define LCD_DC_PIN              PC4

//        LCD_CS auf GND legen

  /*  ------------------------------------------------------------
                         Displayaufloesung
      ------------------------------------------------------------ */

  #define _xres                   128
  #define _yres                   128

  #define mirror                  0                 // 0 : normale Ausgabe
                                                    // 1 : Spiegelbildausgabe

    /*  ------------------------------------------------------------
                         Setupflags fuer SPI-Displays
        ------------------------------------------------------------ */

    // fuer Berechnung Bildadressen. ACHTUNG: manche Chinadisplays behandeln 128x128 Displays
    // so, als haette es 160 Pixel in Y-Aufloesung.

    // In diesem Fall ist fuer _lcyofs  -32 anzugeben
    // (hat nur Effekt, wenn _yres   128 , im Hauptprogramm dann outmode= 3; damit das Bild
    // nicht af dem Kopf steht)

    #define tft128                  2                 // 1: Display ohne Offset (aelter)
                                                      // 2: Display mit Offset (neuer)

    #define  tft128_g2              1                 // neuere TFT 128x128 haben eine
                                                      // abweichende Colum-Start und Row-Start
                                                      // 0 : ohne Abweichung
                                                      // 1 : mit Abweichung


    #define rgbseq                  1                 // Reihenfolge der erwarteten Farbuebergabe bei ST7735 LC-Controllern
                                                      // 0: blau-gruen-rot
                                                      // 1: rot-gruen-blau

    #define pindefs                 5                 // unterschiedliche Anschluesse an den Controller
                                                      // Deklarationen der Anschlusspins in tft_pindefs.h
                                                      // 1 : Anschluss Lochrasterboard
                                                      // 2 : Anschluss R3 - Evalboard (gedruckte Schaltung)
                                                      // 3 : Steckbrett
                                                      // 5 : TFT-Button Shield

    #define tft_wait                0                 // 0 = keine Wartefunktion nach spi_out
                                                      // 1 = es wird nach spi_out ein nop eingefuegt


    #define flickerreduce           0                 // 0 : normal
                                                      // 1 : Reduzierung fuer neuere KMR-1.8 SPI Displays

    #define negativout              0                 // 0 : normal
                                                      // 1 : Farben werden invertiert wiedergegeben (fuer ST7789 notwendig)

  /*  ------------------------------------------------------------
        Sonderfall TFT 128x128 / ST7735 Controller 2. Generation
      ------------------------------------------------------------ */
  #if ((tft128_g2 == 1) && (_xres == 128) && (_yres == 128))
    #define colofs                2
    #define rowofs                3
  #else
    #define colofs                0
    #define rowofs                0
  #endif

  /*  ------------------------------------------------------------
        Display-Offset neue/alte 128x128 Pixel TFTs
      ------------------------------------------------------------ */
  #if (tft128 == 2)
    #define _lcyofs               -32               // manche Display sprechen das Display an
                                                    // als haette es 160 Pixel Y-Aufloesung
  #else
    #define _lcyofs               0
  #endif

  /*  ------------------------------------------------------------
       soll innerhalb der fillrect - Funktion ein Fastfill mittels
       der Funktionen des Displays vorgenommen werden (hier
       funktioniert dann ein "Drehen" mittels outmode NICHT)
     ------------------------------------------------------------- */

  #define  fastfillmode             1

  /*  ------------------------------------------------------------
                         P R O T O T Y P E N
      ------------------------------------------------------------ */

  // ----------------- LCD - Benutzerfunktionen ---------------

  void lcd_init(void);                                                        // initialisiert Display
  void putpixel(int x, int y,uint16_t color);                                 // schreibt einen einzelnen Punkt auf das Display
  void clrscr();                                                              // loescht Display-Inhalt
  void fastxline(uint8_t x1, uint8_t y1, uint8_t x2, uint16_t color);         // zeichnet eine Linie in X-Achse
  void fillrect(int x1, int y1, int x2, int y2, uint16_t color);              // fuellt einen rechteckigen Bereich mit Farbe aus
  uint16_t rgbfromvalue(uint8_t r, uint8_t g, uint8_t b);                     // konvertiert einen 24 Bit RGB-Farbwert in einen 16 Bit Farbwert
  uint16_t rgbfromega(uint8_t entry);                                         // konvertiert einen Farbwert aus der EGA-Palette in einen 16 Bit Farbwert
  void gotoxy(unsigned char x, unsigned char y);                              // setzt den Textcursor fuer Textausgaben
  void setfont(uint8_t nr);                                                   // setzt Schriftstil: 0= 8x8 Pixel, 2= 5x7 Pixel
  void lcd_putchar(char ch);                                                  // setzt ein Zeichen auf das Display
  void lcd_putchar5x7(unsigned char ch);
  void lcd_putchar8x8(unsigned char ch);
  void lcd_putchar12x16(unsigned char ch);
  void outtextxy(int x, int y, uint8_t dir, char *dataPtr);                   // gibt einen String an Pixelkoordinaten auf dem LCD aus
  void line(int x0, int y0, int x1, int y1, uint16_t color);                  // zeichnet eine Linie
  void rectangle(int x1, int y1, int x2, int y2, uint16_t color);             // zeichnet ein Rechteck
  void ellipse(int xm, int ym, int a, int b, uint16_t color );                // zeichnet eine Ellipse
  void fillellipse(int xm, int ym, int a, int b, uint16_t color );            // zeichnet eine ausgefuellte Ellipse
  void circle(int x, int y, int r, uint16_t color );                          // zeichnet einen Kreis
  void fillcircle(int x, int y, int r, uint16_t color );                      // zeichnet einen ausgefuellten Kreis
  void showimage(int ox, int oy, const unsigned char* const image, uint16_t fwert);     // zeichnet ein monochromes Bitmap
  void putstring(char *c);                                                    // schreibt einen Textstring auf das LCD

  // ---------------- Low-level Displayfunktionen -------------

  void set_ram_address (uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2);  // setzt den zu beschreibenden Speicherbereich
  void setcol(int startcol);                    // setzt zu beschreibende Spalte
  void setpage(int startpage);                  // setzt zu beschreibende Reihe
  void setxypos(int x, int y);                  // setzt die zu beschreibende Koordinate im Display-Ram

  // --------------------- SPI-Funktionen ---------------------

  void wrcmd(uint8_t cmd);                      // schreibt einzelnes Kommandodatum (Registerzugriff)
  void wrdata(uint8_t data);                    // schreibt einzelnen Registerwert oder Ramwert
  void wrdata16(int data);                      // schreibt einen Integerwert

  /*  ------------------------------------------------------------
                         Displayaufloesung
      ------------------------------------------------------------ */

  #define _xres                   128
  #define _yres                   128

  /*  ------------------------------------------------------------
                      EGA - Farbzuweisungen
      ------------------------------------------------------------ */

  #define black                   0
  #define blue                    1
  #define green                   2
  #define cyan                    3
  #define red                     4
  #define magenta                 5
  #define brown                   6
  #define grey                    7
  #define darkgrey                8
  #define lightblue               9
  #define lightgreen              10
  #define lightcyan               11
  #define lightred                12
  #define lightmagenta            13
  #define yellow                  14
  #define white                   15


  extern int      aktxp;                    // Beinhaltet die aktuelle Position des Textcursors in X-Achse
  extern int      aktyp;                    // dto. fuer die Y-Achse
  extern uint16_t textcolor;                // Beinhaltet die Farbwahl fuer die Vordergrundfarbe
  extern uint16_t bkcolor;                  // dto. fuer die Hintergrundfarbe
  extern uint8_t  outmode;                  // Variable zum "Drehen" der Displayausgabe
  extern uint8_t  textsize;                 // Skalierung der Ausgabeschriftgroesse
  extern uint8_t  fontsizex;                // benoetigte Variable zur Positionierung von gotoxy x Achse
  extern uint8_t  fontsizey;                // dto. y Achse

  const extern uint16_t egapalette [];      // RGB565 Farbwerte der EGA-Palette


  //-------------------------------------------------------------
  // Registerzuordnung der Adressierungsregister der
  // verschiedenen Displaycontroller
  //-------------------------------------------------------------

  #define coladdr      0x2a
  #define rowaddr      0x2b
  #define writereg     0x2c


#endif
