/* -----------------------------------------------------------
                            max7219.c

   Softwaremodul zur Ansteuerung des Multpilex IC MAX7219
   in Verbindung mit einer 8x8 LED Matrix (China-Modul)


   Benoetigte Hardware;

            - 8x8 LED Matrix mit MAX7219 IC

   MCU   :  STM8S103F3
   Takt  :  externer Takt 16 MHz


   29.08.17 by R. Seelig

   Pins und Anschlussbelegung siehe max7219.h
   ----------------------------------------------------------- */

#include "max7219.h"


/* -----------------------------------------------------------
     serout

     serielle Datenausgabe mittels Bitbanging, MSB zuerst
   ----------------------------------------------------------- */
void serout(uint8_t data)
{
  uint8_t i, val;

  for(i= 8; i> 0; i--)
  {
    val = (data & (1 << (i-1)) ) >> (i - 1);
    m7219_clkclr();
    if(val)
      m7219_dinset();
    else
      m7219_dinclr();
    m7219_clkset();
  }
}

/* -----------------------------------------------------------
     m7219_send

     Datenuebertragung zu einem MAX7219

        reg  : zu beschreibendes Register des MAX7219
        data : Wert der in das Register des MAX7219
               geschrieben wird.
   ----------------------------------------------------------- */
void m7219_send(uint8_t reg, uint8_t data)
{
  m7219_loadclr();
  serout(reg);
  serout(data);
  m7219_loadset();
}

/* -----------------------------------------------------------
     m7219_init

     Setzt die Anschluesse an die der MAX7219 angeschlossen ist
     als Ausgaenge und intitalisiert die Register des ICs
   ----------------------------------------------------------- */
void m7219_init()
{
  m7219_init_con();

  m7219_dinclr();
  m7219_clkclr();
  m7219_loadclr();

  m7219_send(0x0b, 0x07);   // Scanlimit
  m7219_send(0x09, 0x00);   // LED-Matrix, keine 7 Segment
  m7219_send(0x0c, 0x01);   // Power an
  m7219_send(0x0f, 0x00);   // kein Displaytest
  m7219_send(0x0a, 0x02);   // Intensitaet

  m7219_clrscr();
}

/* -----------------------------------------------------------
     m7219_col

     Setzt einen 8 Bit-Wert ( = 8 "Pixel") in eine Spalte
     der LED-Matrix.

        digit : Spalte, die beschrieben werden soll
        data  : Wert, mit der die Spalte beschrieben wird
                (8 Pixel)
   ----------------------------------------------------------- */
void m7219_col(uint8_t digit, uint8_t data)
{
  m7219_send(digit + 1, data);
}

/* -----------------------------------------------------------
     m7219_clrscr

     loescht die gesamte Matrixanzeige
   ----------------------------------------------------------- */
void m7219_clrscr()
{
  uint8_t i;

  for(i= 0; i< 8; i++)
  {
    m7219_col(i, 0);
  }
}

/* -----------------------------------------------------------
     m7219_setbmp

     Zeichnet ein gesamtes 8x8 Pixel grosses "Bitmap* auf die
     LED-Matrix.

        *bmp  : Zeiger auf 8 Bytes ( = 64 Bits) grossen
                Buffer. Dieses Bitmap wird angezeigt.
   ----------------------------------------------------------- */
void m7219_setbmp(uint8_t *bmp)
{
  uint8_t i;

  for (i= 0; i< 8; i++)
  {
    m7219_col(i, bmp[i]);
  }
}
