/* ----------------------------------------------------------
                          smallio.c

     Sehr schlankes I/O Interface ueber die serielle
     Schnittstelle mit abgespecktem printf. Hierdurch ist es
     moeglich, die Beispiele der Hilfedatei fuer viele
     Mikrocontroller und auf PC nachzuvollziehen.

     Fuer die unterstuetzten Microcontroller (AVR, STM32,
     MCS-51, STM8) ist jeweils ein eigenes smallio
     verfuegbar, sodass JEDES Programm lediglich im Header
     ein

                  "include smallio.h"

     eingefuegt werden muss. Im Makefile ist einzustellen,
     dass smallio.o fuer AVR,ARM und PC-Konsolenprogramme
     oder smallio.rel fuer  MCS-51 und STM8) hinzugelinkt
     werden muss. Eintrag im Makefile:

     SRCS      += ../src/smallio.o

     Die eingestellte Baudrate betraegt bei allen
     Controllern 19200 bd (Ausnahme MCS-51, hier dann
     4800 bd)

     Protokoll ist 8N1

     ------------------------------------------------------

     Von smallio Unterstuetzte Controller in der
     STM8 - Version:

     STM8S003, S103, S105

     03.02.2020  R. Seelig
   ---------------------------------------------------------- */


#include "smallio.h"

#define echo_enable    0

char printfkomma = 1;


  #if (use_usart == 1)

    #define USARTx_SR           USART1_SR
    #define USARTx_DR           USART1_DR
    #define USARTx_BRR1         USART1_BRR1
    #define USARTx_BRR2         USART1_BRR2
    #define USARTx_CR1          USART1_CR1
    #define USARTx_CR2          USART1_CR2
    #define USARTx_CR3          USART1_CR3
    #define USARTx_CR4          USART1_CR4
    #define USARTx_CR5          USART1_CR5
    #define USARTx_GTR          USART1_GTR
    #define USARTx_PSCR         USART1_PSCR

  #endif


  #if (use_usart == 2)

    #define USARTx_SR           USART2_SR
    #define USARTx_DR           USART2_DR
    #define USARTx_BRR1         USART2_BRR1
    #define USARTx_BRR2         USART2_BRR2
    #define USARTx_CR1          USART2_CR1
    #define USARTx_CR2          USART2_CR2
    #define USARTx_CR3          USART2_CR3
    #define USARTx_CR4          USART2_CR4
    #define USARTx_CR5          USART2_CR5
    #define USARTx_GTR          USART2_GTR
    #define USARTx_PSCR         USART2_PSCR

  #endif


/* ------------------------------------------------------------
                       UART-Funktionen
   ------------------------------------------------------------ */

   uint16_t  baudrate;

/* ------------------------------------------------------------
                             putchar

      sendet ein Zeichen auf der seriellen Schnittstelle
   ------------------------------------------------------------ */
void putchar(char ch)
{
  while(!(USARTx_SR & USART_SR_TXE));   // warten bis letztes Zeichen gesendet ist
  USARTx_DR = ch;                       // Zeichen senden
}

/* ------------------------------------------------------------
                            getchar

     wartet so lange, bis auf der seriellen Schnittstelle
     ein Zeichen eintrifft und liest dieses ein
   ------------------------------------------------------------ */
char getchar(void)
{
  while(!(USARTx_SR & USART_SR_RXNE));    // solange warten bis ein Zeichen eingetroffen ist
  return USARTx_DR;
}

/* ------------------------------------------------------------
                         keypressed
      testet, ob auf der seriellen Schnittstelle ein Zeichen
      zum Lesen ansteht (liest dieses aber nicht und wartet
      auch nicht darauf)
   ------------------------------------------------------------ */
char keypressed(void)
{
  return (USARTx_SR & USART_SR_RXNE);
}

/* ------------------------------------------------------------
                         putstring

     sendet einen durch den Zeiger *p adressierten
     AsciiZ-String auf der seriellen Schnittstelle
   ------------------------------------------------------------ */
void uart_putstring(char *p)
{
  do
  {
    putchar( *p );
  } while( *p++);
}


/* ------------------------------------------------------------
                            uart_init

      Initialisieren der seriellen Schnittstelle. Mit internem
      Taktgeber funktioniert die Schnittstelle nur bis
      57600 Baud
   ------------------------------------------------------------ */
void uart_init(void)
{

  // Taktinitialisierung erfolgt hier, damit im MAIN ausser einem
  // uart_init() nichts gemacht werden muss, damit das System
  // funktioniert

  PD_DDR |= (1 << PD4);                                 // TxD Leitung als Output
  PD_CR1 |= (1 << PD4);

  USARTx_CR2 |= USART_CR2_TEN;                          // TxD enabled
  USARTx_CR2 |= USART_CR2_REN;                          // RxD enable
  USARTx_CR3 &= ~(USART_CR3_STOP1);                     // Stopbit

  // baudratendivisor:
  //   Divisior = F_CPU / baudrate
  //   Nibbles des Divisors N3 N2 N1 N0
  //   Bsp.:   16 MHz / 9600 bd = 1667d = 0683h
  //      Hier waere dann:
  //        N3 = 0   / N2 = 6  / N1 = 8  / N0 = 3

  //      BRR1 bekaeme N2 und N1 => 0x68
  //      BRR2 bekaeme N3 und N0 => 0x03

  //  USARTx_BRR1 = 0x68; // 9600 Baud
  //  USARTx_BRR2 = 0x03;

  if (baudrate == 57600)
  {
    USARTx_BRR1 = 0x11;
    USARTx_BRR2 = 0x06;
  }
  if (baudrate == 38400)
  {
    USARTx_BRR1 = 0x1a;
    USARTx_BRR2 = 0x0a;
  }
  if (baudrate == 28800)
  {
    USARTx_BRR1 = 0x22;
    USARTx_BRR2 = 0x0c;
  }
  if (baudrate == 19200)
  {
    USARTx_BRR1 = 0x34;
    USARTx_BRR2 = 0x02;
  }
  if (baudrate == 9600)
  {
    USARTx_BRR1 = 0x68;
    USARTx_BRR2 = 0x03;
  }
  if (baudrate == 4800)
  {
    USARTx_BRR1 = 0xd0;
    USARTx_BRR2 = 0x05;
  }
}


#if (readint_enable == 1)

  /* ------------------------------------------------------------
                              readint

       liest einen 16-Bit signed Integer auf der seriellen
       Schnittstelle ein (allerdings reicht der Eingabebereich
       nur von -32767 .. +32767).

       Korrektur ist mit der Loeschtaste nach links moeglich.
     ------------------------------------------------------------ */
  int16_t readint(void)
  {
    uint8_t   signflag= 0;
    uint16_t  sum = 0;
    uint8_t   zif;
    uint8_t   ch;

    do
    {
      ch= getchar();
      if (ch== 0x0a) ch= 0x0d;

      // Ziffern auswerten
      if ((ch>= '0') && (ch<= '9'))
      {
        zif= ch-'0';

        if ((sum== 0) && (zif))        // erste Ziffer
        {
          sum= zif;
          putchar(ch);
        }
        else
        if (sum < 3277)
        {
          if  (!( ((sum * 10) > 32750) && (zif > 7) ))
          {
            sum= (sum*10) + zif;
            putchar(ch);
          }
        }
      }

      // letzte Eingabe loeschen
      if ((ch== 127) || (ch== 8))    // letzte Eingabe loeschen
      {
        if (sum)
        {
          sum /= 10;
          putchar(8);
          putchar(' ');
          putchar(8);
        }
        else
        if (signflag)
        {
          putchar(8);
          putchar(' ');
          putchar(8);
          signflag= 0;
        }
      }

      // Eingabe Minuszeichen
      if ((ch== '-') && (sum == 0))
      {
        signflag= 1;
          putchar('-');
      }

    } while (ch != 0x0d);              // wiederholen bis Returnzeichen eintrifft
    if (signflag) return sum *(-1); else return sum;
  }

#endif

/* ------------------------------------------------------------
                         printf - Funktionen
   ------------------------------------------------------------ */

/* ------------------------------------------------------------
                            putint

     gibt einen Integer dezimal aus. Ist Uebergabe
     "komma" != 0 wird ein "Kommapunkt" mit ausgegeben.

     Bsp.: 12345 wird als 123.45 ausgegeben.
     (ermoeglicht Pseudofloatausgaben im Bereich)
   ------------------------------------------------------------ */
void putint(int16_t i, char komma)
{
  typedef enum boolean { FALSE, TRUE }bool_t;

  static uint16_t zz[]      = { 10000, 1000, 100, 10 };
  bool_t          not_first = FALSE;

  uint8_t    zi;
  int16_t    z, b;

  komma= 5-komma;

  if (!i)
  {
    putchar('0');
  }
  else
  {
    if(i < 0)
    {
      putchar('-');
      i = -i;
    }

    for(zi = 0; zi < 4; zi++)
    {
      z = 0;
      b = 0;

      if  ((zi== komma) && komma)
      {
        if (!not_first) putchar('0');
        putchar('.');
        not_first= TRUE;
      }

      while(z + zz[zi] <= i)
      {
        b++;
        z += zz[zi];
      }
      if(b || not_first)
      {
        putchar('0' + b);
        not_first = TRUE;
      }
      i -= z;
    }
    if (komma== 4) putchar('.');
    putchar('0' + i);
  }
}

/* ------------------------------------------------------------
                          hexnibbleout

     gibt die unteren 4 Bits eines chars als Hexaziffer aus.
     Eine Pruefung ob die oberen vier Bits geloescht sind
     erfolgt NICHT !
  -------------------------------------------------------------  */
void hexnibbleout(uint8_t b)
{
  if (b< 10) b+= '0'; else b+= 55;
  putchar(b);
}

/* ------------------------------------------------------------
                            puthex

     gibt einen Integer hexadezimal aus. Ist die auszugebende
     Zahl >= 0xff erfolgt die Ausgabe 2-stellig, ist sie
     groesser erfolgt die Ausgabe 4-stellig.
   ------------------------------------------------------------ */
void puthex(uint16_t h)
{
  uint8_t b;

  if (h> 0xff)                    // 16 Bit-Wert
  {
    b= (h >> 12);
    hexnibbleout(b);
    b= (h >> 8) & 0x0f;
    hexnibbleout(b);
  }
  b= h;
  b= (h >> 4) & 0x0f;
  hexnibbleout(b);
  b= h & 0x0f;
  hexnibbleout(b);
}

void putstring(char *p)
{
  do
  {
    putchar( *p );
  } while( *p++);
}


/* ------------------------------------------------------------
                             my_printf

     alternativer Ersatz fuer printf.

     Aufruf:

         my_printf("Ergebnis= %d",zahl);

     Platzhalterfunktionen:

        %s     : Ausgabe Textstring
        %d     : dezimale Ausgabe
        %x     : hexadezimale Ausgabe
                 ist Wert > 0xff erfolgt 4-stellige
                 Ausgabe
                 is Wert <= 0xff erfolgt 2-stellige
                 Ausgabe
        %k     : Integerausgabe als Pseudokommazahl
                 12345 wird als 123.45 ausgegeben
        %c     : Ausgabe als Asciizeichen

   ------------------------------------------------------------ */
void my_printf(const char *s,...)
{
  int       arg1;
  char      *arg2;
  uint8_t   token;
  char      ch;
  va_list   ap;

  va_start(ap,s);
  do
  {
    ch= *s;
    if(ch== 0) return;

    if(ch=='%')            // Platzhalterzeichen
    {
      s++;
      token= *s;
      switch(token)
      {
        case 'd':          // dezimale Ausgabe
        {
          arg1= va_arg(ap,int);
          putint(arg1,0);
          break;
        }
        case 'x':          // hexadezimale Ausgabe
        {
          arg1= va_arg(ap,int);
          puthex(arg1);
          break;
        }
        case 'k':
        {
          arg1= va_arg(ap,int);
          putint(arg1,printfkomma);     // Integerausgabe mit Komma: 12896 zeigt 12.896 an
          break;
        }
        case 'c':          // Zeichenausgabe
        {
          arg1= va_arg(ap,int);
          putchar(arg1);
          break;
        }
        case '%':
        {
          putchar(token);
          break;
        }
        case 's':
        {
          arg2= va_arg(ap,char *);
          putstring(arg2);
          break;
        }
      }
    }
    else
    {
      putchar(ch);
    }
    s++;
  }while (ch != '\0');
}

/* ------------------------------------------------------
                   System - Funktionen
   ------------------------------------------------------ */

/* ------------------------------------------------------
                     sysclock_init
      stellt den Taktgeber fuer den Controller ein.
      Bei Verwendung des internen Oszillators werden
      16MHz eingestellt

      (Xtal enable) xtalen = 1  => externer Quarz
                           = 0  => interner RC-Oszillator

      Anmerkung:
         soll der MCU mit externem Quarz laufen, ist
         zuerst ein Aufruf

            sysclock_init(0);

         gefolgt von einem Aufruf

            sysclock_init(1);

         zu erfolgen !
   ------------------------------------------------------ */
void sysclock_init(char xtalen)
{
  if (xtalen)
  {
    CLK_ICKR = 0;                                  //  Reset Register interner clock
    CLK_ECKR = HSEEN;

    while ((CLK_ECKR & HSERDY) == 0);              //  warten bis Quarz eingeschwungen ist;

    CLK_SWR = 0xb4;                                //  int. Generator als Taktquelle
//    while ((CLK_SWCR &  SWIF) != 0);               //  warten bis Takt stabil
    CLK_SWCR = SWEN;                               //  Enable switching.

    CLK_CKDIVR = 0;                                //  Taktteiler auf volle Geschwindigkeit
    CLK_PCKENR1 = 0xff;                            //  alle Peripherietakte an
    CLK_PCKENR2 = 0xff;                            //  dto.
  }
  else
  {
    CLK_ICKR = 0;                                  //  Reset Register interner clock
    CLK_ECKR = 0;                                  //  Reset Register externer clock (ext. clock disable)

    CLK_ICKR =  HSIEN;                             //  Interner clock enable
    while ((CLK_ICKR & (HSIRDY)) == 0);            //  warten bis int. Takt eingeschwungen ist


    CLK_CKDIVR = 0;                                //  Taktteiler auf volle Geschwindigkeit
    CLK_PCKENR1 = 0xff;                            //  alle Peripherietakte an
    CLK_PCKENR2 = 0xff;                            //  dto.

    CLK_CCOR = 0;                                  //  CCO aus
    CLK_HSITRIMR = 0;                              //  keine Taktjustierung
    CLK_SWIMCCR = 0;                               //  SWIM = clock / 2.
    CLK_SWR = 0xe1;                                //  int. Generator als Taktquelle
    CLK_SWCR = 0;                                  //  Reset clock switch control register.
    CLK_SWCR = SWEN;                               //  Enable switching.
    while ((CLK_SWCR &  SWBSY) != 0);              //  warten bis Peripherietakt stabil
  }
  delay(50);
}

/* ------------------------------------------------------
                          delay
     grobe Zeitverzoegerung in Millisekunden. Der
     Wert 1435 in cnt2 ist "ausgemessen" und stimmt nur
     fuer eine Taktfrequenz von 16Mhz so "halbwegs". Fuer
     eine genauere Zeit ist besser eine Interruptroutine
     mittels Timer zu verwenden.
  ------------------------------------------------------ */
void delay(uint16_t cnt)
{
  volatile uint16_t cnt2;

  while (cnt)
  {
    cnt2= 1435;
    while(cnt2)
    {
      cnt2--;
    }
    cnt--;
  }
}

/* ------------------------------------------------------
                        delay_us
     grobe Zeitverzoegerung in Mikrosekunden. Fuer
     genauere Zeitverzoegerung  ist besser eine Interrupt-
     routine mittels Timer zu verwenden.
  ------------------------------------------------------ */
void delay_us(uint16_t anz)
{
  volatile uint16_t count;

  for (count= 0; count< anz; count++)
  {
    __asm
      nop
    __endasm;
  }
}

/* ------------------------------------------------------
                     int_enable
      grundsaetzlich Interrupts zulassen

      ( hier definiert, weil man diese Funktion bei
      jedem Programm benoetigt welches mit Interrupts
      arbeitet und somit nicht jedesmal implementiert
      werden muss und zudem nur sehr wenige Bytes Speicher
      benoetigt)
   ------------------------------------------------------ */
void int_enable(void)
{
  __asm;
    rim
  __endasm;
}

/* ------------------------------------------------------
                     int_disable
      grundsaetzlich Interrupts sperren

      ( hier definiert, weil man diese Funktion bei
      jedem Programm benoetigt welches mit Interrupts
      arbeitet und somit nicht jedesmal implementiert
      werden muss und zudem nur sehr wenige Bytes Speicher
      benoetigt)
   ------------------------------------------------------ */
void int_disable(void)
{
  __asm;
    sim
  __endasm;
}


/* ------------------------------------------------------
                       smallio_init

    initialisiert die Taktfrequenz und die serielle
    Schnittstelle
   ------------------------------------------------------ */
void smallio_init()
{
  sysclock_init(0);
  delay(4);
  baudrate= BAUDRATE;
  uart_init();
}
